" Vim syntax file
" Language:    FORTH (qartus flavour)
" Designer:    Christian V. J. Brssow <cvjb@cvjb.de>
" Maintainer:  Antonio A. Maschio                    
" Last Update: 25 gennaio 2020
" Filenames:   *.ft

" $Id: qartus.vim,v 1.14 2012/05/27 15:57:22 bruessow Exp $

" For version 5.x: Clear all syntax items
" For version 6.x: Quit when a syntax file was already loaded
if version < 600
    syntax clear
elseif exists("b:current_syntax")
    finish
endif

let s:cpo_save = &cpo
set cpo&vim
set ts=3

" Synchronization method
syn sync ccomment
syn sync maxlines=200

" I use qartus, so I set this to case ignore
syn case ignore

" Some special, non-FORTH keywords
syn keyword qartusTodo contained TODO FIXME XXX

" Characters allowed in keywords
if version >= 600
    setlocal iskeyword=!,@,33-35,%,$,38-64,A-Z,91-96,a-z,123-126
else
    set iskeyword=!,@,33-35,%,$,38-64,A-Z,91-96,a-z,123-126
endif

" Keywords

" basic mathematical and logical operators
syn keyword qartusOperators + - * / MOD /MOD NEGATE ABS MIN MAX 
syn keyword qartusOperators AND OR XOR NOT SHIFT 1+
syn keyword qartusOperators 1- 2+ 2- U* D* D/ UD/
syn keyword qartusOperators */ */MOD  U/MOD U/
syn keyword qartusOperators D+ D- DNEGATE DABS DMIN DMAX
syn keyword qartusOperators 0< 0= 0> < = > U< 
syn keyword qartusOperators D0= D< D= DU<  

" stack manipulations
syn keyword qartusStack DROP DUP OVER SWAP ROT ?DUP -DUP PICK ROLL
syn keyword qartusStack 2DROP 2DUP 2OVER 2SWAP 2ROT R0 S0
syn keyword qartusRStack >R R> R@ CSP !CSP ?CSP CF-MOVE

" address operations
syn keyword qartusMemory @ ! C@ C! C, 2@ 2! . D. D.R DEPTH DUMP U. U.R UD. .R
syn keyword qartusMemory 32@ 32! 32, ? H.
syn keyword qartusAdrArith ALLOT , +! 
syn keyword qartusMemBlks MOVE CMOVE FILL

" conditionals
syn keyword qartusCond IF ELSE THEN CASE OF ENDOF ENDCASE ENDIF
syn keyword qartusCond (IF) (ELSE) (THEN) (CASE) (OF) (ENDOF) (ENDCASE)

" iterations
syn keyword qartusLoop BEGIN WHILE REPEAT UNTIL AGAIN END
syn keyword qartusLoop (BEGIN) (WHILE) (REPEAT) (UNTIL) (AGAIN)
syn keyword qartusLoop LOOP I J DO +LOOP /LOOP
syn keyword qartusLoop (LOOP) (DO) (+LOOP) (/LOOP)
syn keyword qartusLoop LEAVE EXIT 

" defining, compiling and executing words
syn keyword qartusEndOfColonDef : ; 
syn match qartusDefine "\'"
syn match qartusDefine '\['
syn match qartusDefine ']'
syn keyword qartusDefine CONSTANT 2CONSTANT VARIABLE 2VARIABLE <BUILDS
syn keyword qartusDefine CREATE DOES> IMMEDIATE EXEC-ONLY COMPILE-ONLY
syn keyword qartusDefine COMPILE EXECUTE FIND (SLITERAL) (COMPILE) (DOES>)
syn keyword qartusDefine LITERAL STATE LAST DLITERAL
syn match qartusDefine   "\[COMPILE\]"

" basic character operations
syn keyword qartusCharOps ASCII EXPECT SPAN WORD TYPE -TRAILING EMIT KEY
syn keyword qartusCharOps TIB CR #TIB >IN COUNT QUERY QUIT SPACE SPACES BL
syn keyword qartusCharOps ?TERMINAL PAGE OUT (WORD) ON OFF

" char-number conversion
syn keyword qartusConversion <# # #> #S CONVERT 
syn keyword qartusConversion DPL FLD HOLD SIGN 

" user error
syn keyword qartusUser ERROR ?ERROR WARNING MESSAGE ERR

" interpreter, debugger
syn keyword qartusForth BYE COLD ABORT (ABORT) HERE 79-STANDARD SEE
syn keyword qartusForth PAD WORDS FORGET FENCE INFO LOAD-SYSTEM SAVE-SYSTEM
syn keyword qartusForth ON-DEBUG OFF-DEBUG INCLUDE NOOP BELL BEEP (SOUND) 
syn keyword qartusForth DATE TIME PC .S CLOCK MCLOCK REQUIRE ENSURE INTERPRET
syn keyword qartusForth WHERE TRACE BANNER VERSION FIG-STATE

" vocabularies
syn keyword qartusVocs FORTH ORDER CONTEXT CURRENT
syn keyword qartusVocs VOCABULARY DEFINITIONS #VOCS (VOCABULARY)

" File/block keywords
syn keyword qartusBlocksAndFiles (BLOCK-READ) (BLOCK-WRITE) FIRST
syn keyword qartusBlocksAndFiles LOAD --> LIST SCR BLK BLOCK BFR BUFFER
syn keyword qartusBlocksAndFiles EMPTY-BUFFERS UPDATE  OFFSET C/L L/SCR
syn keyword qartusBlocksAndFiles SAVE-BUFFERS B/BUF EDITOR EDIT B/SCR
syn keyword qartusBlocksAndFiles #BUFFERS SEARCH PORT LISTEN SEND
syn keyword qartusBlocksAndFiles CLOSE-FILE OPEN-FILE R/O 
syn keyword qartusBlocksAndFiles READ-LINE W/O WRITE-LINE

" Editor words
syn keyword qartusEditor X TOP TILL TEXT T STAMP S R P N MARK M 
syn keyword qartusEditor  L I H F E D COPY CLEAR C B A #L #P R#

" numbers
syn keyword qartusMath DECIMAL BASE HEX TRUE FALSE
syn match qartusInteger '\<-\=[0-9.]*[0-9.]\+\>'
syn match qartusInteger '\<&-\=[0-9.]*[0-9.]\+\>'
" recognize hex and binary numbers, the '$' and '%' notation is for gforth
syn match qartusInteger '\<\$\x*\x\+\>' " *1* --- dont't mess
syn match qartusInteger '\<\x*\d\x*\>'  " *2* --- this order!
syn match qartusInteger '\<%[0-1]*[0-1]\+\>'

" Strings
syn region qartusString start=+\.\"+ end=+"+ end=+$+ 

" \ Comment
syn match qartusComment '\\\s.*$' contains=qartusTodo
" ( comment
syn region qartusComment start='\(^\|\s\)\zs(\s' end=')\n' end=')\s' contains=qartusTodo
" (S comment
syn region qartusComment start='\(^\|\s\)\zs(s\s' end=')\n' end=')\s' contains=qartusTodo
" ;S comment
syn region qartusComment start=";S" end="\%$" contains=qartusTodo


" Define the default highlighting.
" For version 5.7 and earlier: only when not done already
" For version 5.8 and later: only when an item doesn't have highlighting yet
if version >= 508 || !exists("did_qartus_syn_inits")
    if version < 508
	let did_qartus_syn_inits = 1
	command -nargs=+ HiLink hi link <args>
    else
	command -nargs=+ HiLink hi def link <args>
    endif

    " The default methods for highlighting. Can be overridden later.
    HiLink qartusTodo Todo
    HiLink qartusOperators Operator
    HiLink qartusMath Number
    HiLink qartusInteger Number
    HiLink qartusFloat Float
    HiLink qartusStack Special
    HiLink qartusRstack Special
    HiLink qartusFStack Special
    HiLink qartusSP Special
    HiLink qartusMemory Function
    HiLink qartusAdrArith Function
    HiLink qartusMemBlks Function
    HiLink qartusCond Conditional
    HiLink qartusLoop Repeat
    HiLink qartusColonDef Define
    HiLink qartusEndOfColonDef Define
    HiLink qartusDefine Define
    HiLink qartusDebug Debug
    HiLink qartusAssembler Include
    HiLink qartusCharOps Character
    HiLink qartusConversion String
    HiLink qartusForth Statement
    HiLink qartusVocs Statement
    HiLink qartusString String
    HiLink qartusComment Comment
    HiLink qartusClassDef Define
    HiLink qartusEndOfClassDef Define
    HiLink qartusObjectDef Define
    HiLink qartusEndOfObjectDef Define
    HiLink qartusInclude Include
    HiLink qartusLocals Type " nothing else uses type and locals must stand out
    HiLink qartusDeprecated Error " if you must, change to Type
    HiLink qartusFileMode Function
    HiLink qartusFileWords Statement
    HiLink qartusBlocksAndFiles Statement
    HiLink qartusEditor Statement
    HiLink qartusUser Error

    delcommand HiLink
endif

let b:current_syntax = "qartus"

let &cpo = s:cpo_save
unlet s:cpo_save
" vim:ts=8:sw=4:nocindent:smartindent:
